/* date_time
 *  
 * A dátum és az idő kijelzése a 2x8 digites PCF8577 LCD kijelzőn. 
 * Az időt és a dátumot egy előre beállított DS3231 RTC-ből
 * olvassuk ki. Azt RTC és a kijelző is I2C buszon kommunikál.
 * 
 * Kapcsolás:
 *    Arduino       DS3231 RTC       PCF8577C LCD 2x8 character 7-segment display
 *   ----------    ------------    -----------------------------------------------~                              ~
 *  |       A4 |--| SDA    SDA |--| SDA     _    _    _    _    _    _    _    _  ~
 *  |       A5 |--| SCL    SCL |--| SCL   /_ / /_ / /_ / /_ / /_ / /_ / /_ / /_ / ~       
 *  |      GND |--| GND    GND |--| GND  /_ /./_ /./_ /./_ /./_ /./_ /./_ /./_ / ~     
 *  |       5V |--| VCC    VCC |--| VCC                                           ~
 *   ----------    ------------    -----------------------------------------------~ 
 */
#include <Wire.h>
#define RTC_ADDRESS 0x68        // A DS3231 RTC I2C címe
#define LCD_ADDRESS 0x3A        // A PCF8577 vezérlő I2C címe

// A szegmensek bitsorrendje: g,f,a,b,e,d,c,DP (<-ez a legalacsonyabb helyiérték)
char digits[10] = {126, 18, 188, 182, 210, 230, 238, 50, 254, 246};
char lcd_data[16];              // A diplay buffer (itt állítjuk össze a képet)
char timeDate[16];              // Ide töltjük a fogadott adatokat

void setup() {
  Wire.begin();                 // I2C csatorna inicializálása
}

void loop() {
  I2C_burstRead(RTC_ADDRESS,0,7,timeDate);   // Dátum és idő beolvasása
  displayDateTime();                         // A kiírandó kép összeállítása
  I2C_burstWrite(LCD_ADDRESS,0,16,lcd_data); // A kijelzés frissítése
  delay(500);                                // várunk fél másodpercet
}

//--- A dátum és az idő beolvasása az RTC-ből --------
void I2C_burstRead(char saddr, char maddr, int size, char* data) {
  Wire.beginTransmission(saddr);             // Start feltétel, slave address küldés
  Wire.write(maddr);                         // Memóriacím kiküldése
  Wire.endTransmission();                    // Írás vége
  Wire.requestFrom(saddr, size);             // Restart és adatok beolvasása
  for (int i = 0; i < size; i++) {           // Az adatok eltárolása
    data[i] = Wire.read();                   // Az beolvasott adatok eltárolása
  }
}

//--- A dátum és az idő kiírása az LCD kijelzőn ------
void I2C_burstWrite(char saddr, char maddr, int size, char* data) {
  Wire.beginTransmission(saddr);             // Start feltétel, slave address küldés
  Wire.write(maddr);                         // Memóriacím kiküldése
  Wire.write(data,16);                       // A szegmensvezérlő adatok kiírása
  Wire.endTransmission();
}

//--- A dátum és az idő kiírása az lcd_data[] tömbbe ------
void displayDateTime() {
  lcd_data[0] = digits[timeDate[0] & 0x0f];        // Másodpercek
  lcd_data[1] = digits[(timeDate[0] >> 4) & 0x07]; // Tízmásodpercek
  lcd_data[2] = 128; // '-'
  lcd_data[3] = digits[timeDate[1] & 0x0f];        // Percek
  lcd_data[4] = digits[(timeDate[1] >> 4) & 0x07]; // Tízpercek
  lcd_data[5] = 128; // '-'
  lcd_data[6] = digits[timeDate[2] & 0x0f];        // Órák
  lcd_data[7] = digits[(timeDate[2] >> 4) & 0x03]; // Tízórák
  lcd_data[8] = digits[timeDate[4] & 0x0f];        // Napok
  lcd_data[9] = digits[(timeDate[4] >> 4) & 0x03]; // Tíznapok
  lcd_data[10] = digits[timeDate[5] & 0x0f] | 1;   // Hónapok + DP
  lcd_data[11] = digits[(timeDate[5] >> 4) & 0x01];// Tízhónapok
  lcd_data[12] = digits[timeDate[6] & 0x0f] | 1;   // Évek + DP
  lcd_data[13] = digits[(timeDate[6] >> 4) & 0x0f];// Évtizedek
  lcd_data[14] = digits[0];                        // Évszázad = 20xx
  lcd_data[15] = digits[2];
}
