/* rtc_baremetal
 *  
 * DS3231 RTC beállítása és kijelzése soros porton. 
 * A beállításhoz a setTime() hívásában szereplő bemenő
 * adatokban adhatjuk meg az aktuális időt 
 * sec, min, hour, dow, day, month, year sorrendben
 * pl: setTime(00,15,15,1,3,2,20) => 2020-FEB-03 15:15:00
 * 
 * Huzalozás:
 *   --------           ------------
 *  |Arduino |         | DS3231 RTC | 
 *  |     A5 |---------| SCL        |  
 *  |     A4 |---------| SDA        |
 *  |     5V |---------| VCC        |
 *  |    GND |---------| GND        |
 *   --------           ------------
 * 
 */
#include "Wire.h"
#define RTC_ADDRESS 0x68
char daysOfTheWeek[7][12] = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday","Sunday"};
 
void setup() {
  Wire.begin();
  Serial.begin(9600);
  //--- set time as: seconds, minutes, hours, day, date, month, year
  setTime(00,15,15,1,3,2,20);        // 2020-FEB-03, 15:15:00
}

void loop() {
  displayTime();                     // display time/date
  delay(1000);                       // every second
}

//--- Convert decimal numbers to BCD -----------------------
byte decToBcd(byte val) {
  return( (val/10*16) + (val%10) );
}

//--- Convert BCD numbers to decimal  ----------------------
byte bcdToDec(byte val) {
  return( (val/16*10) + (val%16) );
}

//--- Set RTC time/date ------------------------------------
void setTime(byte second, byte minute, byte hour, 
     byte dayOfWeek, byte dayOfMonth, byte month, byte year) {
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0);                     // set register pointer to 00h
  Wire.write(decToBcd(second));      // set seconds
  Wire.write(decToBcd(minute));      // set minutes
  Wire.write(decToBcd(hour));        // set hours
  Wire.write(decToBcd(dayOfWeek));   // set day of week (1=Sunday, 7=Saturday)
  Wire.write(decToBcd(dayOfMonth));  // set date (1 to 31)
  Wire.write(decToBcd(month));       // set month
  Wire.write(decToBcd(year));        // set year (0 to 99)
  Wire.endTransmission();
}

//--- Read time/date from RTC ------------------------------
void getTime(byte *second, byte *minute, byte *hour, 
     byte *dayOfWeek, byte *dayOfMonth, byte *month, byte *year) {
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0);                     // set register pointer to 00h
  Wire.endTransmission();
  Wire.requestFrom(RTC_ADDRESS, 7);  // request 7 bytes starting from register 00h
  *second = bcdToDec(Wire.read() & 0x7f);
  *minute = bcdToDec(Wire.read());
  *hour = bcdToDec(Wire.read() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.read());
  *dayOfMonth = bcdToDec(Wire.read());
  *month = bcdToDec(Wire.read());
  *year = bcdToDec(Wire.read());
}

void displayTime() { 
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  // retrieve data from DS3231
  getTime(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
  Serial.print(hour, DEC);
  Serial.print(":");
  if (minute < 10) Serial.print("0");
  Serial.print(minute, DEC);
  Serial.print(":");
  if (second < 10) Serial.print("0");
  Serial.print(second, DEC);
  Serial.print(" ");
  Serial.print(year+2000, DEC);
  Serial.print("/");
  Serial.print(month, DEC);
  Serial.print("/");
  Serial.print(dayOfMonth, DEC);
  Serial.print(" ");
  Serial.println(daysOfTheWeek[dayOfWeek-1]); 
}
